library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
use IEEE.std_logic_unsigned.all;


--Library UNISIM;
--use UNISIM.vcomponents.all;

LIBRARY lpm;
USE lpm.lpm_components.ALL;


library work;
use work.malib.all;

entity top2 is port(

clk50 , init, init_tiny :in std_logic;
d,g : in std_logic;
hsynch , vsynch : out std_logic;
vga_blank_n ,vgaclk , vga_sync_n : out std_logic;
rgb:out std_logic_vector(2 downto 0)
);
end entity;
architecture behavior of top2 is

constant black : std_logic_vector(2 downto 0) := "000";
constant blue : std_logic_vector(2 downto 0) := "001"; 
constant green  : std_logic_vector(2 downto 0) := "010";
constant cyan : std_logic_vector(2 downto 0) := "011";
constant red : std_logic_vector(2 downto 0) := "100";
constant magenta : std_logic_vector(2 downto 0) := "101";
constant yellow : std_logic_vector(2 downto 0) := "110";
constant white : std_logic_vector(2 downto 0) := "111";

--Registres et PORTs de l'ATTiny861
constant OCR1A : std_logic_vector(5 downto 0) := "101101";
constant OCR1B : std_logic_vector(5 downto 0) := "101100";
constant PORTA : std_logic_vector(5 downto 0) := "011011";
constant DDRA : std_logic_vector(5 downto 0) := "011010";
constant PINA : std_logic_vector(5 downto 0) := "011001";
constant PORTB : std_logic_vector(5 downto 0) := "011000";
constant DDRB : std_logic_vector(5 downto 0) := "010111";
constant PINB : std_logic_vector(5 downto 0) := "010110";
constant ADCH : std_logic_vector(5 downto 0) := "000101";
constant ADCL : std_logic_vector(5 downto 0) := "000100";
--Registres non présents dans l'ATTiny861
constant UDR : std_logic_vector(5 downto 0) := "000011";
constant UCSRA : std_logic_vector(5 downto 0) := "000010";
constant UCSRB : std_logic_vector(5 downto 0) := "000001";
constant TCNT1 : std_logic_vector(5 downto 0) := "101110"; -- 0x2e
constant TCCR1B : std_logic_vector(5 downto 0) := "101111"; -- 0x2f
	component mcu_core is
		Port (
			Clk	: in std_logic;
			Rst	: in std_logic; -- Reset core when Rst='1'
			En		: in std_logic; -- CPU stops when En='0', could be used to slow down cpu to save power
			-- PM
			PM_A		: out std_logic_vector(15 downto 0);
			PM_Drd	: in std_logic_vector(15 downto 0);
			-- DM
			DM_A		: out std_logic_vector(15 downto 0); -- 0x00 - xxxx
			DM_Areal	: out std_logic_vector(15 downto 0); -- 0x60 - xxxx (same as above + io-adr offset)
			DM_Drd	: in std_logic_vector(7 downto 0);
			DM_Dwr	: out std_logic_vector(7 downto 0);
			DM_rd		: out std_logic;
			DM_wr		: out std_logic;
			-- IO
			IO_A		: out std_logic_vector(5 downto 0); -- 0x00 - 0x3F
			IO_Drd	: in std_logic_vector(7 downto 0);
			IO_Dwr	: out std_logic_vector(7 downto 0);
			IO_rd		: out std_logic;
			IO_wr		: out std_logic;
			-- OTHER
		   OT_FeatErr	: out std_logic; -- Feature error! (Unhandled part of instruction)
		   OT_InstrErr	: out std_logic -- Instruction error! (Unknown instruction)
		);
	end component mcu_core;
	--PM
	component pm  is
		Port (
				Clk	: in std_logic;
				rst	: in std_logic; -- Reset when Rst='1'
				-- PM
				PM_A		: in std_logic_vector(15 downto 0);
				PM_Drd	: out std_logic_vector(15 downto 0)
		);
	end component pm;	

  component dm is
    Port ( clk : in  STD_LOGIC;
           addr : in  STD_LOGIC_VECTOR (15 downto 0);
           dataread : out  STD_LOGIC_VECTOR (7 downto 0);
           datawrite : in  STD_LOGIC_VECTOR (7 downto 0);
           rd : in  STD_LOGIC;
           wr : in  STD_LOGIC);
  end component dm;


type bitmap is array (7 downto 0) of std_logic_vector(7 downto 0 );
--constant ball : bitmap := ( x"3c",x"7e",x"ff", x"ff",x"ff",x"ff", x"7e",x"3c");
constant ball : bitmap := ( x"00",x"3c",x"7e", x"7e",x"7e",x"7e", x"3c",x"00");

signal clk25 ,video_on: std_logic;
signal pixel_x , pixel_y : std_logic_vector(9 downto 0);
signal rgbraq,rgbbric1,rgbball, rgbbord : std_logic_vector(2 downto 0);
signal pixball : std_logic;
signal q : std_logic_vector(31 downto 0);

signal ADDR : std_logic_vector(10 downto 0);
signal DO : std_logic_vector(7 downto 0);
signal val_x, val_y ,ballx, bally: std_logic_vector(9 downto 0);
signal ligne , colonne: std_logic_vector(9 downto 0);
signal DIP : std_logic_vector (0 downto 0);

signal raqx,raqy: std_logic_vector(9 downto 0);
signal ligneball : std_logic_vector(7 downto 0);
-- signaux intermediaires processeur
signal rst : std_logic;
signal lost_msb : std_logic_vector(5 downto 0);
signal PM_A		: std_logic_vector(15 downto 0);
	signal PM_Drd	: std_logic_vector(15 downto 0);
	-- DM
	signal DM_A			: std_logic_vector(15 downto 0); -- 0x00 - xxxx
	signal DM_Areal	: std_logic_vector(15 downto 0); -- 0x60 - xxxx (same as above + io-adr offset)
	signal DM_Drd		: std_logic_vector(7 downto 0);
	signal DM_Dwr		: std_logic_vector(7 downto 0);
	signal DM_rd		: std_logic;
	signal DM_wr		: std_logic;
	-- IO
	signal IO_A		: std_logic_vector(5 downto 0); -- 0x00 - 0x3F
	signal IO_Drd	: std_logic_vector(7 downto 0);
	signal IO_Dwr	: std_logic_vector(7 downto 0);
	signal IO_rd	: std_logic;
	signal IO_wr	: std_logic;

	signal IO_DrdA	: std_logic_vector(7 downto 0);
	signal IO_DrdB	: std_logic_vector(7 downto 0);
	
	
	signal vx,vy : std_logic_vector(31 downto 0);
	signal ctrl : std_logic_vector(7 downto 0);
	
	signal ballxinit,ballyinit : std_logic_vector(9 downto 0);
	signal load,en,udx,udy,clkx,clky,resetball : std_logic;
	signal clk_mcu : std_logic;
	
	

begin
--
--process(clk50) begin
--	if rising_edge(clk50) then
--		clk_mcu <= not clk_mcu;
--	end if;
--end process;

vga_blank_n <= '1';
vgaclk <= clk25; -- horoe 25MHz
vga_sync_n <= '0';

clk_mcu <= clk50;

	vx(31 downto 16) <= (others =>'0');
	vy(31 downto 16) <= (others =>'0');

   rst <=not(init_tiny);

	core : mcu_core Port map (
		Clk	=> clk_mcu,
		Rst	=> Rst,
		En		=> '1',
		-- PM
		PM_A		=> PM_A,
		PM_Drd	=> PM_Drd,
		-- DM
		DM_A		=> DM_A,
		DM_Areal	=> DM_Areal,
		DM_Drd	=> DM_Drd,
		DM_Dwr	=> DM_Dwr,
		DM_rd		=> DM_rd,
		DM_wr		=> DM_wr,
		-- IO
		IO_A		=> IO_A,
		IO_Drd	=> IO_Drd,
		IO_Dwr	=> IO_Dwr,
		IO_rd		=> IO_rd,
		IO_wr		=> IO_wr,
		-- OTHER
		OT_FeatErr => open,
		OT_InstrErr	=> open
	);

--	prgmem : pm port map (
--			Clk	=> clk50,
--			Rst	=> Rst,
--			-- PM
--			PM_A		=> PM_A,
--			PM_Drd	=> PM_Drd
--	);
--	
	
	prgmem : LPM_ROM

        generic map(LPM_WIDTH => 16,    -- MUST be greater than 0
                 LPM_WIDTHAD => 16,    -- MUST be greater than 0
                 LPM_ADDRESS_CONTROL  => "REGISTERED",
                 LPM_OUTDATA  => "UNREGISTERED",
                 LPM_FILE => "./soft/chenillar.mif",
                 LPM_TYPE  => L_ROM,
                 INTENDED_DEVICE_FAMILY   => "UNUSED",
                 LPM_HINT  => "UNUSED")

                 port map(ADDRESS => PM_A ,
                           INCLOCK => clk_mcu,
                           MEMENAB =>'1',
                           Q => PM_Drd);

	
	datamem : dm port map (
           clk => clk50,
           addr => DM_A,
           dataread  => DM_Drd,
           datawrite => DM_Dwr,
           rd => DM_rd, 
           wr =>	DM_wr
	);
	
	-- IO write process
--
    iowr: process(CLK50)
    begin
        if (rising_edge(CLK50)) then
            if (IO_wr = '1') then
                case IO_A is
		 -- addresses for tiny861 device (use io.h).
       --
                    when PORTA  => vx(7 downto 0) <= IO_Dwr; 			-- PORTA=X"1B" (0X3B) 
                                   
		              when PORTB => vx(15 downto 8) <= IO_Dwr;	-- PORTB=X"18" (0X3B)
			           				                          
						  when UCSRB => vy(15 downto 8) <=IO_Dwr;			-- PORT UART =X" " ( 0x   )
											 
						  when UCSRA => vy(7 downto 0) <= IO_Dwr;  -- TCNT1		                      
											 
							when UDR => ctrl <= IO_DWR;
						  when others =>
                end case;
            end if;
        end if;
    end process;
	 
-- IO read process
--
    iord: process(IO_rd,IO_A)
    begin
        -- addresses for tinyX6 device (use iom8.h).
        --
		  if IO_rd = '1' then
          case IO_A is
				when PINA => IO_Drd <= raqx(7 downto 0);  -- PINA=X"19" (0X39)
            when PINB => IO_Drd <= "000000"&raqx(9 downto 8);  -- PINB=X"16" (0X36)
            when ADCH => IO_Drd <= "000000"&ballx(9 downto 8);
				when ADCL	=> IO_DRD <= ballx(7 downto 0);
				when TCNT1 => IO_Drd <= "000000"&bally(9 downto 8);
				when TCCR1B	=> IO_DRD <= bally(7 downto 0);
--            when ADCH => IO_DRd <= s_cordic_sin(15 downto 8);
--				when ADCL => IO_DRd <= s_cordic_sin(7 downto 0); -- valeur CRC sur 16 bits
--				when TCCR1B => IO_DRd <= "0000000" & s_cordic_done; -- valeur timer
				when others => IO_Drd <= X"AA";
          end case;
		  end if;
    end process;


-- gene frequence pour deplacement balle
ddsx : dds port map (clk => clk50, dn=> vx , clkout => clkx);
ddsy : dds port map (clk => clk50, dn=> vy , clkout => clky);

load <= ctrl(7);
en <= ctrl(4);
udx <= ctrl(0);
udy <= ctrl(1);

ballxinit <= raqx + "0000010101";
ballyinit <= raqy - "0000001000";
resetball <= load or init_tiny;
-- deplacement balle par compteur
process(clkx,resetball) begin
		if resetball='1' then
			ballx <= ballxinit;
		elsif rising_edge(clkx) then
			if en='1' then
				if udx ='1' then
					if ballx<631 then
						ballx <= ballx +1 ;
					else
						ballx <= "1001110111";
					end if;
				else	-- udx=0
					if ballx > 2 then
						ballx <= ballx -1 ;
					else
						ballx <= "0000000001";
					end if;
				end if;
			end if;
		end if;
end process;

process(clky,resetball) begin
		if resetball='1' then
			bally <= ballyinit;
		elsif rising_edge(clky) then
			if en='1' then
				if udy ='1' then
					if bally<472 then
						bally <= bally +1 ;
					--else
					--	bally <= "0111011000"; 
					end if;
				else	-- udy=0
					if bally > 2 then
						bally <= bally -1 ;
					--else
					--	bally <= "0000000001";
					end if;
				end if;
			end if;
		end if;
end process;
			
			
--genration horloge video 25MHz
process(clk50) begin
    if rising_edge(clk50) then
        clk25<= not clk25;
    end if;
end process;

-- diviseur de frequence
process(clk25) begin
	if rising_edge(clk25) then
		q <= q+1;
	end if;
end process;

-- deplacement raquette
process(q(15)) begin  -- environ 32hz
	if rising_edge(q(15)) then
		if (d='1' and g='0') then
			if raqx<589 then
				raqx <= raqx +1;
			else
				raqx <= "1001001101";  -- 589
			end if;
		elsif (g='1' and d='0') then
			if raqx>0 then
				raqx <= raqx - 1;
			else
				raqx <= (others => '0');
			end if;
		end if;
	end if;
end process;

raqy <= "0110100001";

--synchronistaion VGA
vga : vga_synchro port map(
    clk => clk25,
    hsynch => hsynch,
    vsynch => vsynch,
    pixel_x => pixel_x,
    pixel_y => pixel_y,
	 init=>init,
    video_on => video_on);
    
	 
	 
--gestion pixel ecran

--ballx <= "0001000000";
--bally <= "0010000000";


--process( pixel_y,bally) begin
--	if (pixel_y >= bally and pixel_y< bally+7 ) then 
--		ligne <= pixel_y - bally;
--	else
--		ligne <= (others=>'0');
--	end if;
--end process;
--
--process( pixel_x, ballx) begin
--	if (pixel_x>ballx and pixel_x<ballx+7) then
--		colonne <= pixel_x - ballx;
--	else
--		colonne <=  (others => '0');
--	end if;
--end process;


--ligneball <= ball( unsigned(ligne(2 downto 0)))  ;
--pixball <= ligneball( unsigned(colonne(2 downto 0))) ;

ligne <= pixel_y - bally;
colonne <= pixel_x - ballx;

ligneball <= ball( conv_integer(ligne(2 downto 0)) ) ;

pixball <= ligneball(conv_integer(colonne(2 downto 0)) ) ;
 



bords: bord port map(
	pixel_x => pixel_x, pixel_y => pixel_y, video_on => video_on,
	onoff => '1',
	coloron => white, coloroff => black, rgb => rgbbord);

balle: rec port map (
	pixel_x => pixel_x, pixel_y=>pixel_y, video_on => video_on,
	onoff=> pixball, 
	x => ballx, dx =>"0000001000", y=> bally, dy=> "0000001000",  -- ???
	coloron => white, coloroff=> black, rgb=> rgbball);

raquette:  rec port map (
	pixel_x=> pixel_x, pixel_y=> pixel_y, video_on => video_on,
	onoff => '1',
	x => raqx, dx=> "0000110010", y=> raqy, dy=> "0000001010",
	coloron=> white, coloroff=> black, rgb=> rgbraq );
	
ADDR <= pixel_y(8 downto 3)&pixel_x(9 downto	5);

val_x <= pixel_x(9 downto 5)&"00000";
val_y <= pixel_y(9 downto 3)&"000";

bric1:  rec port map (
	pixel_x=> pixel_x, pixel_y=> pixel_y, video_on => video_on,
	onoff=> DO(0),
	x => val_x, dx=> "0000011111", y=> val_y, dy=> "0000000111",
	coloron=> DO(3 downto 1), coloroff=> black, rgb=> rgbbric1 );
	

	
process(clk25) begin
		if rising_edge(clk25) then
			rgb <= rgbbric1 or rgbraq or rgbball  or rgbbord	;
		end if;
end process;

ram_brique : lpm_ram_dq 
	   GENERIC MAP(LPM_WIDTH => 8,
      --- LPM_TYPE: STRING := L_RAM_DQ,
      LPM_WIDTHAD => 11,
      --LPM_NUMWORDS: STRING := UNUSED,
      LPM_FILE => "ram_brique.mif"
      --LPM_INDATA: STRING := REGISTERED;
      --LPM_ADDRESS_CONTROL => "UNREGISTERED",
      --LPM_OUTDATA => "UNREGISTERED"
		)
	port map (data=>"11111111" ,address=> ADDR, inclock=>clk50, outclock=>clk50,  we =>'0' , q=> DO );
	


--  <-----Cut code below this line and paste into the architecture body---->

   -- RAMB16_S9: 2k x 8 + 1 Parity bit Single-Port RAM
   --            Spartan-3
   -- Xilinx HDL Language Template, version 14.5

--   RAMB16_S9_inst : RAMB16_S9
--   generic map (
--      INIT => X"000", --  Value of output RAM registers at startup
--      SRVAL => X"000", --  Output value upon SSR assertion
--      WRITE_MODE => "WRITE_FIRST", --  WRITE_FIRST, READ_FIRST or NO_CHANGE
--      -- The following INIT_xx declarations specify the initial contents of the RAM
--      -- Address 0 to 511
--      INIT_00 => X"0000000000000000000000000909090909090909090909090909090909090909",
--      INIT_01 => X"0000000000000000000000000D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D",
--      INIT_02 => X"0000000000000000000000000F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F",
--      INIT_03 => X"0000000000000000000000000303030303030303030303030303030303030303",
--      INIT_04 => X"0000000000000000000000000505050505050505050505050505050505050505",
--      INIT_05 => X"0000000000000000000000000707070707070707070707070707070707070707",
--      INIT_06 => X"0000000000000000000000000B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B0B",
--      INIT_07 => X"0000000000000000000000000D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D",
--      INIT_08 => X"0000000000000000000000000303030303030303030303030303030303030303",
--      INIT_09 => X"0000000000000000000000000F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F",
--      INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 512 to 1023
--      INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 1024 to 1535
--      INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 1536 to 2047
--      INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- The next set of INITP_xx are for the parity bits
--      -- Address 0 to 511
--      INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 512 to 1023
--      INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 1024 to 1535
--      INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      -- Address 1536 to 2047
--      INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
--      INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
--   port map (
--      DO => DO,      -- 8-bit Data Output
--      DOP => open,    -- 1-bit parity Output
--      ADDR => ADDR,  -- 11-bit Address Input
--      CLK => CLK50,    -- Clock
--      DI => x"00",      -- 8-bit Data Input
--      DIP => DIP,    -- 1-bit parity Input
--      EN => '1',      -- RAM Enable Input
--      SSR => '0',    -- Synchronous Set/Reset Input
--      WE => '0'       -- Write Enable Input
--   );

end behavior;


